@@ -1,3 +1,4 @@
+#include <array>
 #include <cmath>
 #include <cstddef>
 #include <functional>
@@ -9,11 +10,12 @@
 #include <vector>
 #include <gtest/gtest.h>
 #include <entt/container/dense_set.hpp>
-#include <entt/core/memory.hpp>
+#include <entt/core/bit.hpp>
 #include <entt/core/utility.hpp>
-#include "../common/throwing_allocator.hpp"
-#include "../common/tracked_memory_resource.hpp"
-#include "../common/transparent_equal_to.h"
+#include "../../common/linter.hpp"
+#include "../../common/throwing_allocator.hpp"
+#include "../../common/tracked_memory_resource.hpp"
+#include "../../common/transparent_equal_to.h"
 
 TEST(DenseSet, Functionalities) {
     entt::dense_set<int, entt::identity, test::transparent_equal_to> set;
@@ -48,23 +50,23 @@ TEST(DenseSet, Functionalities) {
     ASSERT_EQ(cset.begin(1u), cset.end(1u));
     ASSERT_EQ(set.cbegin(1u), set.cend(1u));
 
-    ASSERT_FALSE(set.contains(42));
-    ASSERT_FALSE(set.contains(4.2));
+    ASSERT_FALSE(set.contains(64));
+    ASSERT_FALSE(set.contains(6.4));
 
-    ASSERT_EQ(set.find(42), set.end());
-    ASSERT_EQ(set.find(4.2), set.end());
-    ASSERT_EQ(cset.find(42), set.cend());
-    ASSERT_EQ(cset.find(4.2), set.cend());
+    ASSERT_EQ(set.find(64), set.end());
+    ASSERT_EQ(set.find(6.4), set.end());
+    ASSERT_EQ(cset.find(64), set.cend());
+    ASSERT_EQ(cset.find(6.4), set.cend());
 
-    ASSERT_EQ(set.hash_function()(42), 42);
-    ASSERT_TRUE(set.key_eq()(42, 42));
+    ASSERT_EQ(set.hash_function()(64), 64);
+    ASSERT_TRUE(set.key_eq()(64, 64));
 
     set.emplace(0);
 
     ASSERT_EQ(set.count(0), 1u);
-    ASSERT_EQ(set.count(4.2), 0u);
+    ASSERT_EQ(set.count(6.4), 0u);
     ASSERT_EQ(cset.count(0.0), 1u);
-    ASSERT_EQ(cset.count(42), 0u);
+    ASSERT_EQ(cset.count(64), 0u);
 
     ASSERT_FALSE(set.empty());
     ASSERT_EQ(set.size(), 1u);
@@ -111,58 +113,63 @@ TEST(DenseSet, Constructors) {
 
 TEST(DenseSet, Copy) {
     entt::dense_set<std::size_t, entt::identity> set;
-    set.max_load_factor(set.max_load_factor() - .05f);
+    const auto max_load_factor = set.max_load_factor() - .05f;
+    set.max_load_factor(max_load_factor);
     set.emplace(3u);
 
     entt::dense_set<std::size_t, entt::identity> other{set};
 
     ASSERT_TRUE(set.contains(3u));
     ASSERT_TRUE(other.contains(3u));
-    ASSERT_EQ(set.max_load_factor(), other.max_load_factor());
+    ASSERT_EQ(other.max_load_factor(), max_load_factor);
 
-    set.emplace(1u);
-    set.emplace(11u); // NOLINT
-    other.emplace(0u);
+    set.emplace(0u);
+    set.emplace(8u);
+    other.emplace(1u);
     other = set;
 
     ASSERT_TRUE(other.contains(3u));
-    ASSERT_TRUE(other.contains(1u));
-    ASSERT_TRUE(other.contains(11u));
-    ASSERT_FALSE(other.contains(0u));
+    ASSERT_TRUE(other.contains(0u));
+    ASSERT_TRUE(other.contains(8u));
+    ASSERT_FALSE(other.contains(1u));
 
-    ASSERT_EQ(other.bucket(3u), set.bucket(11u));
-    ASSERT_EQ(other.bucket(3u), other.bucket(11u));
-    ASSERT_EQ(*other.begin(3u), *set.begin(3u));
-    ASSERT_EQ(*other.begin(3u), 11u);
-    ASSERT_EQ((*++other.begin(3u)), 3u);
+    ASSERT_EQ(other.bucket(0u), set.bucket(8u));
+    ASSERT_EQ(other.bucket(0u), other.bucket(8u));
+    ASSERT_EQ(*other.begin(0u), *set.begin(0u));
+    ASSERT_EQ(*other.begin(0u), 8u);
+    ASSERT_EQ((*++other.begin(0u)), 0u);
 }
 
 TEST(DenseSet, Move) {
     entt::dense_set<std::size_t, entt::identity> set;
-    set.max_load_factor(set.max_load_factor() - .05f);
+    const auto max_load_factor = set.max_load_factor() - .05f;
+    set.max_load_factor(max_load_factor);
     set.emplace(3u);
 
     entt::dense_set<std::size_t, entt::identity> other{std::move(set)};
 
-    ASSERT_EQ(set.size(), 0u); // NOLINT
+    test::is_initialized(set);
+
+    ASSERT_TRUE(set.empty());
     ASSERT_TRUE(other.contains(3u));
-    ASSERT_EQ(set.max_load_factor(), other.max_load_factor());
+    ASSERT_EQ(other.max_load_factor(), max_load_factor);
 
     set = other;
-    set.emplace(1u);
-    set.emplace(11u); // NOLINT
-    other.emplace(0u);
+    set.emplace(0u);
+    set.emplace(8u);
+    other.emplace(1u);
     other = std::move(set);
+    test::is_initialized(set);
 
-    ASSERT_EQ(set.size(), 0u); // NOLINT
+    ASSERT_TRUE(set.empty());
     ASSERT_TRUE(other.contains(3u));
-    ASSERT_TRUE(other.contains(1u));
-    ASSERT_TRUE(other.contains(11u));
-    ASSERT_FALSE(other.contains(0u));
+    ASSERT_TRUE(other.contains(0u));
+    ASSERT_TRUE(other.contains(8u));
+    ASSERT_FALSE(other.contains(1u));
 
-    ASSERT_EQ(other.bucket(3u), other.bucket(11u));
-    ASSERT_EQ(*other.begin(3u), 11u);
-    ASSERT_EQ(*++other.begin(3u), 3u);
+    ASSERT_EQ(other.bucket(0u), other.bucket(8u));
+    ASSERT_EQ(*other.begin(0u), 8u);
+    ASSERT_EQ(*++other.begin(0u), 0u);
 }
 
 TEST(DenseSet, Iterator) {
@@ -445,12 +452,12 @@ TEST(DenseSet, Insert) {
     ASSERT_EQ(it, --set.end());
     ASSERT_EQ(*it, 3);
 
-    int range[2u]{7, 9}; // NOLINT
+    std::array range{2, 4};
     set.insert(std::begin(range), std::end(range));
 
     ASSERT_EQ(set.size(), 4u);
-    ASSERT_TRUE(set.contains(7));
-    ASSERT_NE(set.find(9), set.end());
+    ASSERT_TRUE(set.contains(2));
+    ASSERT_NE(set.find(4), set.end());
 }
 
 TEST(DenseSet, InsertRehash) {
@@ -612,11 +619,16 @@ TEST(DenseSet, Erase) {
 
     for(std::size_t next{}, last = minimum_bucket_count + 1u; next < last; ++next) {
         ASSERT_TRUE(set.contains(next));
+        ASSERT_EQ(set.bucket(next), next);
+        ASSERT_EQ(set.bucket_size(next), 1u);
     }
 
     auto it = set.erase(++set.begin());
     it = set.erase(it, it + 1);
 
+    ASSERT_EQ(set.bucket_size(1u), 0u);
+    ASSERT_EQ(set.bucket_size(8u), 0u);
+
     ASSERT_EQ(*--set.end(), 6u);
     ASSERT_EQ(set.erase(6u), 1u);
     ASSERT_EQ(set.erase(6u), 0u);
@@ -628,17 +640,6 @@ TEST(DenseSet, Erase) {
     ASSERT_EQ(*it, 7u);
     ASSERT_EQ(*--set.end(), 5u);
 
-    for(std::size_t next{}, last = minimum_bucket_count + 1u; next < last; ++next) {
-        if(next == 1u || next == 8u || next == 6u) { // NOLINT
-            ASSERT_FALSE(set.contains(next));
-            ASSERT_EQ(set.bucket_size(next), 0u);
-        } else {
-            ASSERT_TRUE(set.contains(next));
-            ASSERT_EQ(set.bucket(next), next);
-            ASSERT_EQ(set.bucket_size(next), 1u);
-        }
-    }
-
     set.erase(set.begin(), set.end());
 
     for(std::size_t next{}, last = minimum_bucket_count + 1u; next < last; ++next) {
@@ -676,7 +677,7 @@ TEST(DenseSet, EraseFromBucket) {
     for(std::size_t next{}; next < 4u; ++next) {
         ASSERT_TRUE(set.emplace(2u * minimum_bucket_count * next).second);
         ASSERT_TRUE(set.emplace(2u * minimum_bucket_count * next + 2u).second);
-        ASSERT_TRUE(set.emplace(2u * minimum_bucket_count * (next + 1u) - 1u).second);
+        ASSERT_TRUE(set.emplace(2u * minimum_bucket_count * next + 3u).second);
     }
 
     ASSERT_EQ(set.bucket_count(), 2u * minimum_bucket_count);
@@ -684,7 +685,7 @@ TEST(DenseSet, EraseFromBucket) {
 
     ASSERT_EQ(set.bucket_size(0u), 4u);
     ASSERT_EQ(set.bucket_size(2u), 4u);
-    ASSERT_EQ(set.bucket_size(15u), 4u);
+    ASSERT_EQ(set.bucket_size(3u), 4u);
 
     set.erase(set.end() - 3, set.end());
 
@@ -693,7 +694,7 @@ TEST(DenseSet, EraseFromBucket) {
 
     ASSERT_EQ(set.bucket_size(0u), 3u);
     ASSERT_EQ(set.bucket_size(2u), 3u);
-    ASSERT_EQ(set.bucket_size(15u), 3u);
+    ASSERT_EQ(set.bucket_size(3u), 3u);
 
     for(std::size_t next{}; next < 3u; ++next) {
         ASSERT_TRUE(set.contains(2u * minimum_bucket_count * next));
@@ -702,31 +703,31 @@ TEST(DenseSet, EraseFromBucket) {
         ASSERT_TRUE(set.contains(2u * minimum_bucket_count * next + 2u));
         ASSERT_EQ(set.bucket(2u * minimum_bucket_count * next + 2u), 2u);
 
-        ASSERT_TRUE(set.contains(2u * minimum_bucket_count * (next + 1u) - 1u));
-        ASSERT_EQ(set.bucket(2u * minimum_bucket_count * (next + 1u) - 1u), 15u);
+        ASSERT_TRUE(set.contains(2u * minimum_bucket_count * next + 3u));
+        ASSERT_EQ(set.bucket(2u * minimum_bucket_count * next + 3u), 3u);
     }
 
     ASSERT_FALSE(set.contains(2u * minimum_bucket_count * 3u));
     ASSERT_FALSE(set.contains(2u * minimum_bucket_count * 3u + 2u));
-    ASSERT_FALSE(set.contains(2u * minimum_bucket_count * (3u + 1u) - 1u));
+    ASSERT_FALSE(set.contains(2u * minimum_bucket_count * 3u + 3u));
 
     set.erase(*++set.begin(0u));
     set.erase(*++set.begin(2u));
-    set.erase(*++set.begin(15u)); // NOLINT
+    set.erase(*++set.begin(3u));
 
     ASSERT_EQ(set.bucket_count(), 2u * minimum_bucket_count);
     ASSERT_EQ(set.size(), 6u);
 
     ASSERT_EQ(set.bucket_size(0u), 2u);
     ASSERT_EQ(set.bucket_size(2u), 2u);
-    ASSERT_EQ(set.bucket_size(15u), 2u);
+    ASSERT_EQ(set.bucket_size(3u), 2u);
 
     ASSERT_FALSE(set.contains(2u * minimum_bucket_count * 1u));
     ASSERT_FALSE(set.contains(2u * minimum_bucket_count * 1u + 2u));
-    ASSERT_FALSE(set.contains(2u * minimum_bucket_count * (1u + 1u) - 1u));
+    ASSERT_FALSE(set.contains(2u * minimum_bucket_count * 1u + 3u));
 
-    while(set.begin(15) != set.end(15u)) { // NOLINT
-        set.erase(*set.begin(15));         // NOLINT
+    while(set.begin(3) != set.end(3u)) {
+        set.erase(*set.begin(3));
     }
 
     ASSERT_EQ(set.bucket_count(), 2u * minimum_bucket_count);
@@ -734,7 +735,7 @@ TEST(DenseSet, EraseFromBucket) {
 
     ASSERT_EQ(set.bucket_size(0u), 2u);
     ASSERT_EQ(set.bucket_size(2u), 2u);
-    ASSERT_EQ(set.bucket_size(15u), 0u);
+    ASSERT_EQ(set.bucket_size(3u), 0u);
 
     ASSERT_TRUE(set.contains(0u * minimum_bucket_count));
     ASSERT_TRUE(set.contains(0u * minimum_bucket_count + 2u));
@@ -749,7 +750,7 @@ TEST(DenseSet, EraseFromBucket) {
 
     ASSERT_EQ(set.bucket_size(0u), 1u);
     ASSERT_EQ(set.bucket_size(2u), 1u);
-    ASSERT_EQ(set.bucket_size(15u), 0u);
+    ASSERT_EQ(set.bucket_size(3u), 0u);
 
     ASSERT_FALSE(set.contains(0u * minimum_bucket_count));
     ASSERT_TRUE(set.contains(0u * minimum_bucket_count + 2u));
@@ -895,13 +896,13 @@ TEST(DenseSet, Rehash) {
     ASSERT_TRUE(set.contains(32u));
     ASSERT_EQ(set.bucket(32u), 0u);
 
-    set.rehash(12u); // NOLINT
+    set.rehash(minimum_bucket_count + 1u);
 
     ASSERT_EQ(set.bucket_count(), 2u * minimum_bucket_count);
     ASSERT_TRUE(set.contains(32u));
     ASSERT_EQ(set.bucket(32u), 0u);
 
-    set.rehash(44u); // NOLINT
+    set.rehash(4u * minimum_bucket_count + 1u);
 
     ASSERT_EQ(set.bucket_count(), 8u * minimum_bucket_count);
     ASSERT_TRUE(set.contains(32u));
@@ -925,7 +926,7 @@ TEST(DenseSet, Rehash) {
     ASSERT_EQ(set.bucket_count(), 2u * minimum_bucket_count);
     ASSERT_TRUE(set.contains(32u));
 
-    set.rehash(55u); // NOLINT
+    set.rehash(4u * minimum_bucket_count + 4u);
 
     ASSERT_EQ(set.bucket_count(), 8u * minimum_bucket_count);
     ASSERT_TRUE(set.contains(32u));
