@@ -1,3 +1,4 @@
+#include <array>
 #include <iterator>
 #include <type_traits>
 #include <utility>
@@ -9,9 +10,9 @@
 #include <entt/entity/registry.hpp>
 #include <entt/entity/snapshot.hpp>
 #include <entt/signal/sigh.hpp>
-#include "../common/config.h"
-#include "../common/empty.h"
-#include "../common/pointer_stable.h"
+#include "../../common/config.h"
+#include "../../common/empty.h"
+#include "../../common/pointer_stable.h"
 
 struct shadow {
     entt::entity target{entt::null};
@@ -50,23 +51,24 @@ TEST(BasicSnapshot, GetEntityType) {
 
     ASSERT_EQ(data.size(), 2u);
 
-    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(&data[0u]), nullptr);
+    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(data.data()), nullptr);
     ASSERT_EQ(entt::any_cast<typename traits_type::entity_type>(data[0u]), storage.size());
 
     ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(&data[1u]), nullptr);
     ASSERT_EQ(entt::any_cast<typename traits_type::entity_type>(data[1u]), storage.free_list());
 
-    entt::entity entity[3u]; // NOLINT
+    constexpr auto number_of_entities = 3u;
+    std::array<entt::entity, number_of_entities> entity{};
 
-    registry.create(std::begin(entity), std::end(entity));
+    registry.create(entity.begin(), entity.end());
     registry.destroy(entity[1u]);
 
     data.clear();
-    snapshot.get<entt::entity>(archive, "ignored"_hs);
+    snapshot.get<entt::entity>(archive);
 
     ASSERT_EQ(data.size(), 5u);
 
-    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(&data[0u]), nullptr);
+    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(data.data()), nullptr);
     ASSERT_EQ(entt::any_cast<typename traits_type::entity_type>(data[0u]), storage.size());
 
     ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(&data[1u]), nullptr);
@@ -94,8 +96,8 @@ TEST(BasicSnapshot, GetType) {
     std::array<entt::entity, number_of_entities> entity{};
     const std::array value{1, 2, 3};
 
-    registry.create(std::begin(entity), std::end(entity));
-    registry.insert<int>(std::begin(entity), std::end(entity), std::begin(value));
+    registry.create(entity.begin(), entity.end());
+    registry.insert<int>(entity.begin(), entity.end(), value.begin());
     registry.destroy(entity[1u]);
 
     std::vector<entt::any> data{};
@@ -105,7 +107,7 @@ TEST(BasicSnapshot, GetType) {
 
     ASSERT_EQ(data.size(), 1u);
 
-    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(&data[0u]), nullptr);
+    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(data.data()), nullptr);
     ASSERT_EQ(entt::any_cast<typename traits_type::entity_type>(data[0u]), 0u);
 
     data.clear();
@@ -113,7 +115,7 @@ TEST(BasicSnapshot, GetType) {
 
     ASSERT_EQ(data.size(), 5u);
 
-    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(&data[0u]), nullptr);
+    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(data.data()), nullptr);
     ASSERT_EQ(entt::any_cast<typename traits_type::entity_type>(data[0u]), storage.size());
 
     ASSERT_NE(entt::any_cast<entt::entity>(&data[1u]), nullptr);
@@ -139,7 +141,7 @@ TEST(BasicSnapshot, GetPointerStableType) {
     constexpr auto number_of_entities = 3u;
 
     std::array<entt::entity, number_of_entities> entity{};
-    const std::array<test::pointer_stable, number_of_entities> value{1, 2, 3};
+    const std::array value{test::pointer_stable{1}, test::pointer_stable{2}, test::pointer_stable{3}};
 
     registry.create(entity.begin(), entity.end());
     registry.insert<test::pointer_stable>(entity.begin(), entity.end(), value.begin());
@@ -152,7 +154,7 @@ TEST(BasicSnapshot, GetPointerStableType) {
 
     ASSERT_EQ(data.size(), 1u);
 
-    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(&data[0u]), nullptr);
+    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(data.data()), nullptr);
     ASSERT_EQ(entt::any_cast<typename traits_type::entity_type>(data[0u]), 0u);
 
     data.clear();
@@ -160,7 +162,7 @@ TEST(BasicSnapshot, GetPointerStableType) {
 
     ASSERT_EQ(data.size(), 6u);
 
-    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(&data[0u]), nullptr);
+    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(data.data()), nullptr);
     ASSERT_EQ(entt::any_cast<typename traits_type::entity_type>(data[0u]), storage.size());
 
     ASSERT_NE(entt::any_cast<entt::entity>(&data[1u]), nullptr);
@@ -186,11 +188,12 @@ TEST(BasicSnapshot, GetEmptyType) {
     entt::registry registry;
     const entt::basic_snapshot snapshot{registry};
     const auto &storage = registry.storage<test::empty>();
+    constexpr auto number_of_entities = 3u;
 
-    entt::entity entity[3u]; // NOLINT
+    std::array<entt::entity, number_of_entities> entity{};
 
-    registry.create(std::begin(entity), std::end(entity));
-    registry.insert<test::empty>(std::begin(entity), std::end(entity));
+    registry.create(entity.begin(), entity.end());
+    registry.insert<test::empty>(entity.begin(), entity.end());
     registry.destroy(entity[1u]);
 
     std::vector<entt::any> data{};
@@ -200,7 +203,7 @@ TEST(BasicSnapshot, GetEmptyType) {
 
     ASSERT_EQ(data.size(), 1u);
 
-    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(&data[0u]), nullptr);
+    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(data.data()), nullptr);
     ASSERT_EQ(entt::any_cast<typename traits_type::entity_type>(data[0u]), 0u);
 
     data.clear();
@@ -208,7 +211,7 @@ TEST(BasicSnapshot, GetEmptyType) {
 
     ASSERT_EQ(data.size(), 3u);
 
-    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(&data[0u]), nullptr);
+    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(data.data()), nullptr);
     ASSERT_EQ(entt::any_cast<typename traits_type::entity_type>(data[0u]), storage.size());
 
     ASSERT_NE(entt::any_cast<entt::entity>(&data[1u]), nullptr);
@@ -229,27 +232,27 @@ TEST(BasicSnapshot, GetTypeSparse) {
     std::array<entt::entity, number_of_entities> entity{};
     const std::array value{1, 2, 3};
 
-    registry.create(std::begin(entity), std::end(entity));
-    registry.insert<int>(std::begin(entity), std::end(entity), std::begin(value));
+    registry.create(entity.begin(), entity.end());
+    registry.insert<int>(entity.begin(), entity.end(), value.begin());
     registry.destroy(entity[1u]);
 
     std::vector<entt::any> data{};
     auto archive = [&data](auto &&elem) { data.emplace_back(std::forward<decltype(elem)>(elem)); };
 
-    snapshot.get<int>(archive, std::begin(entity), std::end(entity), "other"_hs);
+    snapshot.get<int>(archive, entity.begin(), entity.end(), "other"_hs);
 
     ASSERT_EQ(data.size(), 1u);
 
-    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(&data[0u]), nullptr);
+    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(data.data()), nullptr);
     ASSERT_EQ(entt::any_cast<typename traits_type::entity_type>(data[0u]), 0u);
 
     data.clear();
-    snapshot.get<int>(archive, std::begin(entity), std::end(entity));
+    snapshot.get<int>(archive, entity.begin(), entity.end());
 
     ASSERT_EQ(data.size(), 6u);
 
-    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(&data[0u]), nullptr);
-    ASSERT_EQ(entt::any_cast<typename traits_type::entity_type>(data[0u]), static_cast<typename traits_type::entity_type>(std::distance(std::begin(entity), std::end(entity))));
+    ASSERT_NE(entt::any_cast<typename traits_type::entity_type>(data.data()), nullptr);
+    ASSERT_EQ(entt::any_cast<typename traits_type::entity_type>(data[0u]), static_cast<typename traits_type::entity_type>(std::distance(entity.begin(), entity.end())));
 
     ASSERT_NE(entt::any_cast<entt::entity>(&data[1u]), nullptr);
     ASSERT_EQ(entt::any_cast<entt::entity>(data[1u]), entity[0u]);
@@ -303,7 +306,7 @@ TEST(BasicSnapshotLoader, GetEntityType) {
 
     std::vector<entt::any> data{};
     auto archive = [&data, pos = 0u](auto &elem) mutable { elem = entt::any_cast<std::remove_reference_t<decltype(elem)>>(data[pos++]); };
-    const entt::entity entity[3u]{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u), traits_type::construct(1u, 1u)}; // NOLINT
+    const std::array entity{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u), traits_type::construct(1u, 1u)};
 
     ASSERT_FALSE(registry.valid(entity[0u]));
     ASSERT_FALSE(registry.valid(entity[1u]));
@@ -328,7 +331,7 @@ TEST(BasicSnapshotLoader, GetEntityType) {
     data.emplace_back(entity[1u]);
     data.emplace_back(entity[2u]);
 
-    loader.get<entt::entity>(archive, "ignored"_hs);
+    loader.get<entt::entity>(archive);
 
     ASSERT_TRUE(registry.valid(entity[0u]));
     ASSERT_TRUE(registry.valid(entity[1u]));
@@ -354,15 +357,15 @@ TEST(BasicSnapshotLoader, GetType) {
 
     std::vector<entt::any> data{};
     auto archive = [&data, pos = 0u](auto &elem) mutable { elem = entt::any_cast<std::remove_reference_t<decltype(elem)>>(data[pos++]); };
-    const entt::entity entity[2u]{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)}; // NOLINT
-    const int values[2u]{1, 3};                                                                    // NOLINT
+    const std::array entity{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)};
+    const std::array value{1, 3};
 
     ASSERT_FALSE(registry.valid(entity[0u]));
     ASSERT_FALSE(registry.valid(entity[1u]));
 
     data.emplace_back(static_cast<typename traits_type::entity_type>(1u));
     data.emplace_back(entity[0u]);
-    data.emplace_back(values[0u]);
+    data.emplace_back(value[0u]);
 
     loader.get<int>(archive, "other"_hs);
 
@@ -375,10 +378,10 @@ TEST(BasicSnapshotLoader, GetType) {
     data.emplace_back(static_cast<typename traits_type::entity_type>(2u));
 
     data.emplace_back(entity[0u]);
-    data.emplace_back(values[0u]);
+    data.emplace_back(value[0u]);
 
     data.emplace_back(entity[1u]);
-    data.emplace_back(values[1u]);
+    data.emplace_back(value[1u]);
 
     loader.get<int>(archive);
 
@@ -388,8 +391,8 @@ TEST(BasicSnapshotLoader, GetType) {
     ASSERT_EQ(storage.size(), 2u);
     ASSERT_TRUE(storage.contains(entity[0u]));
     ASSERT_TRUE(storage.contains(entity[1u]));
-    ASSERT_EQ(storage.get(entity[0u]), values[0u]);
-    ASSERT_EQ(storage.get(entity[1u]), values[1u]);
+    ASSERT_EQ(storage.get(entity[0u]), value[0u]);
+    ASSERT_EQ(storage.get(entity[1u]), value[1u]);
 }
 
 TEST(BasicSnapshotLoader, GetEmptyType) {
@@ -402,7 +405,7 @@ TEST(BasicSnapshotLoader, GetEmptyType) {
 
     std::vector<entt::any> data{};
     auto archive = [&data, pos = 0u](auto &elem) mutable { elem = entt::any_cast<std::remove_reference_t<decltype(elem)>>(data[pos++]); };
-    const entt::entity entity[2u]{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)}; // NOLINT
+    const std::array entity{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)};
 
     ASSERT_FALSE(registry.valid(entity[0u]));
     ASSERT_FALSE(registry.valid(entity[1u]));
@@ -443,8 +446,8 @@ TEST(BasicSnapshotLoader, GetTypeSparse) {
 
     std::vector<entt::any> data{};
     auto archive = [&data, pos = 0u](auto &elem) mutable { elem = entt::any_cast<std::remove_reference_t<decltype(elem)>>(data[pos++]); };
-    const entt::entity entity[2u]{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)}; // NOLINT
-    const int values[2u]{1, 3};                                                                    // NOLINT
+    const std::array entity{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)};
+    const std::array value{1, 3};
 
     ASSERT_FALSE(registry.valid(entity[0u]));
     ASSERT_FALSE(registry.valid(entity[1u]));
@@ -452,7 +455,7 @@ TEST(BasicSnapshotLoader, GetTypeSparse) {
     data.emplace_back(static_cast<typename traits_type::entity_type>(2u));
     data.emplace_back(static_cast<entt::entity>(entt::null));
     data.emplace_back(entity[0u]);
-    data.emplace_back(values[0u]);
+    data.emplace_back(value[0u]);
 
     loader.get<int>(archive, "other"_hs);
 
@@ -465,12 +468,12 @@ TEST(BasicSnapshotLoader, GetTypeSparse) {
     data.emplace_back(static_cast<typename traits_type::entity_type>(3u));
 
     data.emplace_back(entity[0u]);
-    data.emplace_back(values[0u]);
+    data.emplace_back(value[0u]);
 
     data.emplace_back(static_cast<entt::entity>(entt::null));
 
     data.emplace_back(entity[1u]);
-    data.emplace_back(values[1u]);
+    data.emplace_back(value[1u]);
 
     loader.get<int>(archive);
 
@@ -480,8 +483,8 @@ TEST(BasicSnapshotLoader, GetTypeSparse) {
     ASSERT_EQ(storage.size(), 2u);
     ASSERT_TRUE(storage.contains(entity[0u]));
     ASSERT_TRUE(storage.contains(entity[1u]));
-    ASSERT_EQ(storage.get(entity[0u]), values[0u]);
-    ASSERT_EQ(storage.get(entity[1u]), values[1u]);
+    ASSERT_EQ(storage.get(entity[0u]), value[0u]);
+    ASSERT_EQ(storage.get(entity[1u]), value[1u]);
 }
 
 TEST(BasicSnapshotLoader, GetTypeWithListener) {
@@ -520,8 +523,8 @@ TEST(BasicSnapshotLoader, Orphans) {
 
     std::vector<entt::any> data{};
     auto archive = [&data, pos = 0u](auto &elem) mutable { elem = entt::any_cast<std::remove_reference_t<decltype(elem)>>(data[pos++]); };
-    const entt::entity entity[2u]{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)}; // NOLINT
-    const int value = 42;
+    const std::array entity{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)};
+    const int value = 3;
 
     ASSERT_FALSE(registry.valid(entity[0u]));
     ASSERT_FALSE(registry.valid(entity[1u]));
@@ -572,7 +575,7 @@ TEST(BasicContinuousLoader, GetEntityType) {
 
     std::vector<entt::any> data{};
     auto archive = [&data, pos = 0u](auto &elem) mutable { elem = entt::any_cast<std::remove_reference_t<decltype(elem)>>(data[pos++]); };
-    const entt::entity entity[3u]{traits_type::construct(1u, 0u), traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)}; // NOLINT
+    const std::array entity{traits_type::construct(1u, 0u), traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)};
 
     ASSERT_FALSE(registry.valid(entity[0u]));
     ASSERT_FALSE(registry.valid(entity[1u]));
@@ -605,7 +608,7 @@ TEST(BasicContinuousLoader, GetEntityType) {
     data.emplace_back(entity[1u]);
     data.emplace_back(entity[2u]);
 
-    loader.get<entt::entity>(archive, "ignored"_hs);
+    loader.get<entt::entity>(archive);
 
     ASSERT_TRUE(loader.contains(entity[0u]));
     ASSERT_TRUE(loader.contains(entity[1u]));
@@ -679,7 +682,7 @@ TEST(BasicContinuousLoader, GetEntityType) {
     data.emplace_back(entity[2u]);
     data.emplace_back(entity[0u]);
 
-    loader.get<entt::entity>(archive, "ignored"_hs);
+    loader.get<entt::entity>(archive);
 
     ASSERT_FALSE(loader.contains(entity[0u]));
     ASSERT_TRUE(loader.contains(entity[1u]));
@@ -707,8 +710,8 @@ TEST(BasicContinuousLoader, GetType) {
 
     std::vector<entt::any> data{};
     auto archive = [&data, pos = 0u](auto &elem) mutable { elem = entt::any_cast<std::remove_reference_t<decltype(elem)>>(data[pos++]); };
-    const entt::entity entity[2u]{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)}; // NOLINT
-    const int values[2u]{1, 3};                                                                    // NOLINT
+    const std::array entity{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)};
+    const std::array value{1, 3};
 
     ASSERT_FALSE(loader.contains(entity[0u]));
     ASSERT_FALSE(loader.contains(entity[1u]));
@@ -718,7 +721,7 @@ TEST(BasicContinuousLoader, GetType) {
 
     data.emplace_back(static_cast<typename traits_type::entity_type>(1u));
     data.emplace_back(entity[0u]);
-    data.emplace_back(values[0u]);
+    data.emplace_back(value[0u]);
 
     loader.get<int>(archive, "other"_hs);
 
@@ -734,10 +737,10 @@ TEST(BasicContinuousLoader, GetType) {
     data.emplace_back(static_cast<typename traits_type::entity_type>(2u));
 
     data.emplace_back(entity[0u]);
-    data.emplace_back(values[0u]);
+    data.emplace_back(value[0u]);
 
     data.emplace_back(entity[1u]);
-    data.emplace_back(values[1u]);
+    data.emplace_back(value[1u]);
 
     loader.get<int>(archive);
 
@@ -750,8 +753,8 @@ TEST(BasicContinuousLoader, GetType) {
     ASSERT_EQ(storage.size(), 2u);
     ASSERT_TRUE(storage.contains(loader.map(entity[0u])));
     ASSERT_TRUE(storage.contains(loader.map(entity[1u])));
-    ASSERT_EQ(storage.get(loader.map(entity[0u])), values[0u]);
-    ASSERT_EQ(storage.get(loader.map(entity[1u])), values[1u]);
+    ASSERT_EQ(storage.get(loader.map(entity[0u])), value[0u]);
+    ASSERT_EQ(storage.get(loader.map(entity[1u])), value[1u]);
 }
 
 TEST(BasicContinuousLoader, GetTypeExtended) {
@@ -763,7 +766,7 @@ TEST(BasicContinuousLoader, GetTypeExtended) {
     const auto &storage = registry.storage<shadow>();
 
     std::vector<entt::any> data{};
-    const entt::entity entity[2u]{traits_type::construct(0u, 1u), traits_type::construct(1u, 1u)}; // NOLINT
+    const std::array entity{traits_type::construct(0u, 1u), traits_type::construct(1u, 1u)};
     const shadow value{entity[0u]};
 
     auto archive = [&loader, &data, pos = 0u](auto &elem) mutable {
@@ -817,7 +820,7 @@ TEST(BasicContinuousLoader, GetEmptyType) {
 
     std::vector<entt::any> data{};
     auto archive = [&data, pos = 0u](auto &elem) mutable { elem = entt::any_cast<std::remove_reference_t<decltype(elem)>>(data[pos++]); };
-    const entt::entity entity[2u]{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)}; // NOLINT
+    const std::array entity{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)};
 
     ASSERT_FALSE(loader.contains(entity[0u]));
     ASSERT_FALSE(loader.contains(entity[1u]));
@@ -867,8 +870,8 @@ TEST(BasicContinuousLoader, GetTypeSparse) {
 
     std::vector<entt::any> data{};
     auto archive = [&data, pos = 0u](auto &elem) mutable { elem = entt::any_cast<std::remove_reference_t<decltype(elem)>>(data[pos++]); };
-    const entt::entity entity[2u]{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)}; // NOLINT
-    const int values[2u]{1, 3};                                                                    // NOLINT
+    const std::array entity{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)};
+    const std::array value{1, 3};
 
     ASSERT_FALSE(loader.contains(entity[0u]));
     ASSERT_FALSE(loader.contains(entity[1u]));
@@ -879,7 +882,7 @@ TEST(BasicContinuousLoader, GetTypeSparse) {
     data.emplace_back(static_cast<typename traits_type::entity_type>(2u));
     data.emplace_back(static_cast<entt::entity>(entt::null));
     data.emplace_back(entity[0u]);
-    data.emplace_back(values[0u]);
+    data.emplace_back(value[0u]);
 
     loader.get<int>(archive, "other"_hs);
 
@@ -895,12 +898,12 @@ TEST(BasicContinuousLoader, GetTypeSparse) {
     data.emplace_back(static_cast<typename traits_type::entity_type>(3u));
 
     data.emplace_back(entity[0u]);
-    data.emplace_back(values[0u]);
+    data.emplace_back(value[0u]);
 
     data.emplace_back(static_cast<entt::entity>(entt::null));
 
     data.emplace_back(entity[1u]);
-    data.emplace_back(values[1u]);
+    data.emplace_back(value[1u]);
 
     loader.get<int>(archive);
 
@@ -913,8 +916,8 @@ TEST(BasicContinuousLoader, GetTypeSparse) {
     ASSERT_EQ(storage.size(), 2u);
     ASSERT_TRUE(storage.contains(loader.map(entity[0u])));
     ASSERT_TRUE(storage.contains(loader.map(entity[1u])));
-    ASSERT_EQ(storage.get(loader.map(entity[0u])), values[0u]);
-    ASSERT_EQ(storage.get(loader.map(entity[1u])), values[1u]);
+    ASSERT_EQ(storage.get(loader.map(entity[0u])), value[0u]);
+    ASSERT_EQ(storage.get(loader.map(entity[1u])), value[1u]);
 }
 
 TEST(BasicContinuousLoader, GetTypeWithListener) {
@@ -953,8 +956,8 @@ TEST(BasicContinuousLoader, Orphans) {
 
     std::vector<entt::any> data{};
     auto archive = [&data, pos = 0u](auto &elem) mutable { elem = entt::any_cast<std::remove_reference_t<decltype(elem)>>(data[pos++]); };
-    const entt::entity entity[2u]{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)}; // NOLINT
-    const int value = 42;
+    const std::array entity{traits_type::construct(0u, 0u), traits_type::construct(2u, 0u)};
+    const int value = 3;
 
     ASSERT_FALSE(registry.valid(entity[0u]));
     ASSERT_FALSE(registry.valid(entity[1u]));
