@@ -1,6 +1,6 @@
 #include <algorithm>
+#include <array>
 #include <functional>
-#include <iterator>
 #include <memory>
 #include <tuple>
 #include <type_traits>
@@ -8,13 +8,17 @@
 #include <gtest/gtest.h>
 #include <entt/core/iterator.hpp>
 #include <entt/core/type_info.hpp>
+#include <entt/entity/component.hpp>
 #include <entt/entity/entity.hpp>
 #include <entt/entity/storage.hpp>
-#include "../common/config.h"
-#include "../common/empty.h"
+#include "../../common/config.h"
+#include "../../common/empty.h"
+#include "../../common/linter.hpp"
 
 template<typename Type>
 struct StorageNoInstance: testing::Test {
+    static_assert(entt::component_traits<Type>::page_size == 0u, "Non-empty type not allowed");
+
     using type = Type;
 
     static auto emplace_instance(entt::storage<type> &pool, const entt::entity entt) {
@@ -30,8 +34,8 @@ struct StorageNoInstance: testing::Test {
         if constexpr(std::is_void_v<type>) {
             return pool.insert(from, to);
         } else {
-            const type values[2u]{}; // NOLINT
-            return pool.insert(from, to, std::begin(values));
+            const std::array<type, 2u> value{};
+            return pool.insert(from, to, value.begin());
         }
     }
 
@@ -55,6 +59,7 @@ TYPED_TEST_SUITE(StorageNoInstanceDeathTest, StorageNoInstanceTypes, );
 
 TYPED_TEST(StorageNoInstance, Constructors) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> pool;
 
     ASSERT_EQ(pool.policy(), entt::deletion_policy::swap_and_pop);
@@ -70,71 +75,71 @@ TYPED_TEST(StorageNoInstance, Constructors) {
 
 TYPED_TEST(StorageNoInstance, Move) {
     using value_type = typename TestFixture::type;
-    entt::storage<value_type> pool;
 
-    pool.emplace(entt::entity{3});
+    entt::storage<value_type> pool;
+    const std::array entity{entt::entity{3}, entt::entity{2}};
+
+    pool.emplace(entity[0u]);
 
     static_assert(std::is_move_constructible_v<decltype(pool)>, "Move constructible type required");
     static_assert(std::is_move_assignable_v<decltype(pool)>, "Move assignable type required");
 
     entt::storage<value_type> other{std::move(pool)};
 
-    ASSERT_TRUE(pool.empty()); // NOLINT
+    test::is_initialized(pool);
+
+    ASSERT_TRUE(pool.empty());
     ASSERT_FALSE(other.empty());
 
-    ASSERT_EQ(pool.type(), entt::type_id<value_type>()); // NOLINT
     ASSERT_EQ(other.type(), entt::type_id<value_type>());
-
-    ASSERT_EQ(other.index(entt::entity{3}), 0u);
+    ASSERT_EQ(other.index(entity[0u]), 0u);
 
     entt::storage<value_type> extended{std::move(other), std::allocator<value_type>{}};
 
-    ASSERT_TRUE(other.empty()); // NOLINT
+    test::is_initialized(other);
+
+    ASSERT_TRUE(other.empty());
     ASSERT_FALSE(extended.empty());
 
-    ASSERT_EQ(other.type(), entt::type_id<value_type>()); // NOLINT
     ASSERT_EQ(extended.type(), entt::type_id<value_type>());
-
-    ASSERT_EQ(extended.index(entt::entity{3}), 0u);
+    ASSERT_EQ(extended.index(entity[0u]), 0u);
 
     pool = std::move(extended);
+    test::is_initialized(extended);
 
     ASSERT_FALSE(pool.empty());
-    ASSERT_TRUE(other.empty());    // NOLINT
-    ASSERT_TRUE(extended.empty()); // NOLINT
+    ASSERT_TRUE(other.empty());
+    ASSERT_TRUE(extended.empty());
 
     ASSERT_EQ(pool.type(), entt::type_id<value_type>());
-    ASSERT_EQ(other.type(), entt::type_id<value_type>());    // NOLINT
-    ASSERT_EQ(extended.type(), entt::type_id<value_type>()); // NOLINT
-
-    ASSERT_EQ(pool.index(entt::entity{3}), 0u);
+    ASSERT_EQ(pool.index(entity[0u]), 0u);
 
     other = entt::storage<value_type>{};
-    other.emplace(entt::entity{42}, 42); // NOLINT
+    other.emplace(entity[1u], 2);
     other = std::move(pool);
+    test::is_initialized(pool);
 
-    ASSERT_TRUE(pool.empty()); // NOLINT
+    ASSERT_FALSE(pool.empty());
     ASSERT_FALSE(other.empty());
 
-    ASSERT_EQ(pool.type(), entt::type_id<value_type>()); // NOLINT
     ASSERT_EQ(other.type(), entt::type_id<value_type>());
-
-    ASSERT_EQ(other.index(entt::entity{3}), 0u);
+    ASSERT_EQ(other.index(entity[0u]), 0u);
 }
 
 TYPED_TEST(StorageNoInstance, Swap) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> pool;
     entt::storage<value_type> other;
 
     ASSERT_EQ(pool.type(), entt::type_id<value_type>());
     ASSERT_EQ(other.type(), entt::type_id<value_type>());
 
-    pool.emplace(entt::entity{42}); // NOLINT
+    pool.emplace(entt::entity{4});
 
-    other.emplace(entt::entity{9}); // NOLINT
-    other.emplace(entt::entity{3});
-    other.erase(entt::entity{9}); // NOLINT
+    other.emplace(entt::entity{2});
+    other.emplace(entt::entity{1});
+    other.erase(entt::entity{2});
 
     ASSERT_EQ(pool.size(), 1u);
     ASSERT_EQ(other.size(), 1u);
@@ -147,15 +152,17 @@ TYPED_TEST(StorageNoInstance, Swap) {
     ASSERT_EQ(pool.size(), 1u);
     ASSERT_EQ(other.size(), 1u);
 
-    ASSERT_EQ(pool.index(entt::entity{3}), 0u);
-    ASSERT_EQ(other.index(entt::entity{42}), 0u);
+    ASSERT_EQ(pool.index(entt::entity{1}), 0u);
+    ASSERT_EQ(other.index(entt::entity{4}), 0u);
 }
 
 TYPED_TEST(StorageNoInstance, Getters) {
     using value_type = typename TestFixture::type;
-    entt::storage<value_type> pool;
 
-    pool.emplace(entt::entity{41}, 3); // NOLINT
+    entt::storage<value_type> pool;
+    const entt::entity entity{4};
+
+    pool.emplace(entity, 3);
 
     testing::StaticAssertTypeEq<decltype(pool.get({})), void>();
     testing::StaticAssertTypeEq<decltype(std::as_const(pool).get({})), void>();
@@ -163,45 +170,50 @@ TYPED_TEST(StorageNoInstance, Getters) {
     testing::StaticAssertTypeEq<decltype(pool.get_as_tuple({})), std::tuple<>>();
     testing::StaticAssertTypeEq<decltype(std::as_const(pool).get_as_tuple({})), std::tuple<>>();
 
-    ASSERT_NO_THROW(pool.get(entt::entity{41}));
-    ASSERT_NO_THROW(std::as_const(pool).get(entt::entity{41}));
+    ASSERT_NO_THROW(pool.get(entity));
+    ASSERT_NO_THROW(std::as_const(pool).get(entity));
 
-    ASSERT_EQ(pool.get_as_tuple(entt::entity{41}), std::make_tuple());
-    ASSERT_EQ(std::as_const(pool).get_as_tuple(entt::entity{41}), std::make_tuple());
+    ASSERT_EQ(pool.get_as_tuple(entity), std::make_tuple());
+    ASSERT_EQ(std::as_const(pool).get_as_tuple(entity), std::make_tuple());
 }
 
 ENTT_DEBUG_TYPED_TEST(StorageNoInstanceDeathTest, Getters) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> pool;
+    const entt::entity entity{4};
 
-    ASSERT_DEATH(pool.get(entt::entity{41}), "");
-    ASSERT_DEATH(std::as_const(pool).get(entt::entity{41}), "");
+    ASSERT_DEATH(pool.get(entity), "");
+    ASSERT_DEATH(std::as_const(pool).get(entity), "");
 
-    ASSERT_DEATH([[maybe_unused]] const auto value = pool.get_as_tuple(entt::entity{41}), "");
-    ASSERT_DEATH([[maybe_unused]] const auto value = std::as_const(pool).get_as_tuple(entt::entity{41}), "");
+    ASSERT_DEATH([[maybe_unused]] const auto value = pool.get_as_tuple(entity), "");
+    ASSERT_DEATH([[maybe_unused]] const auto value = std::as_const(pool).get_as_tuple(entity), "");
 }
 
 TYPED_TEST(StorageNoInstance, Value) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> pool;
+    const entt::entity entity{4};
 
-    pool.emplace(entt::entity{42}); // NOLINT
+    pool.emplace(entity);
 
-    ASSERT_EQ(pool.value(entt::entity{42}), nullptr);
+    ASSERT_EQ(pool.value(entt::entity{4}), nullptr);
 }
 
 ENTT_DEBUG_TYPED_TEST(StorageNoInstanceDeathTest, Value) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> pool;
 
-    ASSERT_DEATH([[maybe_unused]] const void *value = pool.value(entt::entity{42}), "");
+    ASSERT_DEATH([[maybe_unused]] const void *value = pool.value(entt::entity{4}), "");
 }
 
 TYPED_TEST(StorageNoInstance, Emplace) {
     using value_type = typename TestFixture::type;
-    entt::storage<value_type> pool;
 
-    const entt::entity entity[2u]{entt::entity{3}, entt::entity{42}}; // NOLINT
+    entt::storage<value_type> pool;
+    const std::array entity{entt::entity{1}, entt::entity{3}};
 
     testing::StaticAssertTypeEq<decltype(pool.emplace({})), void>();
 
@@ -211,9 +223,9 @@ TYPED_TEST(StorageNoInstance, Emplace) {
 
 ENTT_DEBUG_TYPED_TEST(StorageNoInstanceDeathTest, Emplace) {
     using value_type = typename TestFixture::type;
-    entt::storage<value_type> pool;
 
-    const entt::entity entity{42};
+    entt::storage<value_type> pool;
+    const entt::entity entity{4};
 
     testing::StaticAssertTypeEq<decltype(pool.emplace({})), void>();
 
@@ -225,10 +237,10 @@ ENTT_DEBUG_TYPED_TEST(StorageNoInstanceDeathTest, Emplace) {
 
 TYPED_TEST(StorageNoInstance, TryEmplace) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> pool;
     entt::sparse_set &base = pool;
-
-    const entt::entity entity[2u]{entt::entity{3}, entt::entity{42}}; // NOLINT
+    const std::array entity{entt::entity{1}, entt::entity{3}};
 
     ASSERT_NE(this->push_instance(pool, entity[0u]), base.end());
 
@@ -237,15 +249,15 @@ TYPED_TEST(StorageNoInstance, TryEmplace) {
 
     base.erase(entity[0u]);
 
-    ASSERT_NE(base.push(std::begin(entity), std::end(entity)), base.end());
+    ASSERT_NE(base.push(entity.begin(), entity.end()), base.end());
 
     ASSERT_EQ(pool.size(), 2u);
     ASSERT_EQ(base.index(entity[0u]), 0u);
     ASSERT_EQ(base.index(entity[1u]), 1u);
 
-    base.erase(std::begin(entity), std::end(entity));
+    base.erase(entity.begin(), entity.end());
 
-    ASSERT_NE(base.push(std::rbegin(entity), std::rend(entity)), base.end());
+    ASSERT_NE(base.push(entity.rbegin(), entity.rend()), base.end());
 
     ASSERT_EQ(pool.size(), 2u);
     ASSERT_EQ(base.index(entity[0u]), 1u);
@@ -254,9 +266,9 @@ TYPED_TEST(StorageNoInstance, TryEmplace) {
 
 TYPED_TEST(StorageNoInstance, Patch) {
     using value_type = typename TestFixture::type;
-    entt::storage<value_type> pool;
 
-    const entt::entity entity{42};
+    entt::storage<value_type> pool;
+    const entt::entity entity{4};
 
     int counter = 0;
     auto callback = [&counter]() { ++counter; };
@@ -274,6 +286,7 @@ TYPED_TEST(StorageNoInstance, Patch) {
 
 ENTT_DEBUG_TYPED_TEST(StorageNoInstanceDeathTest, Patch) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> pool;
 
     ASSERT_DEATH(pool.patch(entt::null), "");
@@ -281,18 +294,18 @@ ENTT_DEBUG_TYPED_TEST(StorageNoInstanceDeathTest, Patch) {
 
 TYPED_TEST(StorageNoInstance, Insert) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> pool;
+    const std::array entity{entt::entity{1}, entt::entity{3}};
 
-    const entt::entity entity[2u]{entt::entity{3}, entt::entity{42}}; // NOLINT
-
-    pool.insert(std::begin(entity), std::end(entity));
+    pool.insert(entity.begin(), entity.end());
 
     ASSERT_EQ(pool.size(), 2u);
     ASSERT_EQ(pool.index(entity[0u]), 0u);
     ASSERT_EQ(pool.index(entity[1u]), 1u);
 
-    pool.erase(std::begin(entity), std::end(entity));
-    this->insert_instance(pool, std::rbegin(entity), std::rend(entity));
+    pool.erase(entity.begin(), entity.end());
+    this->insert_instance(pool, entity.rbegin(), entity.rend());
 
     ASSERT_EQ(pool.size(), 2u);
     ASSERT_EQ(pool.index(entity[0u]), 1u);
@@ -301,21 +314,20 @@ TYPED_TEST(StorageNoInstance, Insert) {
 
 ENTT_DEBUG_TYPED_TEST(StorageNoInstanceDeathTest, Insert) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> pool;
+    const std::array entity{entt::entity{1}, entt::entity{3}};
 
-    const entt::entity entity[2u]{entt::entity{3}, entt::entity{42}}; // NOLINT
+    pool.insert(entity.begin(), entity.end());
 
-    pool.insert(std::begin(entity), std::end(entity));
-
-    ASSERT_DEATH(pool.insert(std::begin(entity), std::end(entity)), "");
-    ASSERT_DEATH(this->insert_instance(pool, std::begin(entity), std::end(entity)), "");
+    ASSERT_DEATH(pool.insert(entity.begin(), entity.end()), "");
+    ASSERT_DEATH(this->insert_instance(pool, entity.begin(), entity.end()), "");
 }
 
 TYPED_TEST(StorageNoInstance, Iterable) {
     using value_type = typename TestFixture::type;
     using iterator = typename entt::storage<value_type>::iterable::iterator;
 
-    testing::StaticAssertTypeEq<typename iterator::value_type, std::tuple<entt::entity>>();
     testing::StaticAssertTypeEq<typename iterator::value_type, std::tuple<entt::entity>>();
     testing::StaticAssertTypeEq<typename iterator::pointer, entt::input_iterator_pointer<std::tuple<entt::entity>>>();
     testing::StaticAssertTypeEq<typename iterator::reference, typename iterator::value_type>();
@@ -359,7 +371,6 @@ TYPED_TEST(StorageNoInstance, ConstIterable) {
     using value_type = typename TestFixture::type;
     using iterator = typename entt::storage<value_type>::const_iterable::iterator;
 
-    testing::StaticAssertTypeEq<typename iterator::value_type, std::tuple<entt::entity>>();
     testing::StaticAssertTypeEq<typename iterator::value_type, std::tuple<entt::entity>>();
     testing::StaticAssertTypeEq<typename iterator::pointer, entt::input_iterator_pointer<std::tuple<entt::entity>>>();
     testing::StaticAssertTypeEq<typename iterator::reference, typename iterator::value_type>();
@@ -401,6 +412,7 @@ TYPED_TEST(StorageNoInstance, ConstIterable) {
 
 TYPED_TEST(StorageNoInstance, IterableIteratorConversion) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> pool;
 
     pool.emplace(entt::entity{3});
@@ -417,21 +429,22 @@ TYPED_TEST(StorageNoInstance, IterableIteratorConversion) {
 
 TYPED_TEST(StorageNoInstance, IterableAlgorithmCompatibility) {
     using value_type = typename TestFixture::type;
-    entt::storage<value_type> pool;
 
-    pool.emplace(entt::entity{3});
+    entt::storage<value_type> pool;
+    const entt::entity entity{3};
+
+    pool.emplace(entity);
 
     const auto iterable = pool.each();
-    const auto it = std::find_if(iterable.begin(), iterable.end(), [](auto args) { return std::get<0>(args) == entt::entity{3}; });
+    const auto it = std::find_if(iterable.begin(), iterable.end(), [entity](auto args) { return std::get<0>(args) == entity; });
 
-    ASSERT_EQ(std::get<0>(*it), entt::entity{3});
+    ASSERT_EQ(std::get<0>(*it), entity);
 }
 
 TYPED_TEST(StorageNoInstance, ReverseIterable) {
     using value_type = typename TestFixture::type;
     using iterator = typename entt::storage<value_type>::reverse_iterable::iterator;
 
-    testing::StaticAssertTypeEq<typename iterator::value_type, std::tuple<entt::entity>>();
     testing::StaticAssertTypeEq<typename iterator::value_type, std::tuple<entt::entity>>();
     testing::StaticAssertTypeEq<typename iterator::pointer, entt::input_iterator_pointer<std::tuple<entt::entity>>>();
     testing::StaticAssertTypeEq<typename iterator::reference, typename iterator::value_type>();
@@ -475,7 +488,6 @@ TYPED_TEST(StorageNoInstance, ConstReverseIterable) {
     using value_type = typename TestFixture::type;
     using iterator = typename entt::storage<value_type>::const_reverse_iterable::iterator;
 
-    testing::StaticAssertTypeEq<typename iterator::value_type, std::tuple<entt::entity>>();
     testing::StaticAssertTypeEq<typename iterator::value_type, std::tuple<entt::entity>>();
     testing::StaticAssertTypeEq<typename iterator::pointer, entt::input_iterator_pointer<std::tuple<entt::entity>>>();
     testing::StaticAssertTypeEq<typename iterator::reference, typename iterator::value_type>();
@@ -517,6 +529,7 @@ TYPED_TEST(StorageNoInstance, ConstReverseIterable) {
 
 TYPED_TEST(StorageNoInstance, ReverseIterableIteratorConversion) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> pool;
 
     pool.emplace(entt::entity{3});
@@ -533,47 +546,49 @@ TYPED_TEST(StorageNoInstance, ReverseIterableIteratorConversion) {
 
 TYPED_TEST(StorageNoInstance, ReverseIterableAlgorithmCompatibility) {
     using value_type = typename TestFixture::type;
-    entt::storage<value_type> pool;
 
-    pool.emplace(entt::entity{3});
+    entt::storage<value_type> pool;
+    const entt::entity entity{3};
+
+    pool.emplace(entity);
 
     const auto iterable = pool.reach();
-    const auto it = std::find_if(iterable.begin(), iterable.end(), [](auto args) { return std::get<0>(args) == entt::entity{3}; });
+    const auto it = std::find_if(iterable.begin(), iterable.end(), [entity](auto args) { return std::get<0>(args) == entity; });
 
-    ASSERT_EQ(std::get<0>(*it), entt::entity{3});
+    ASSERT_EQ(std::get<0>(*it), entity);
 }
 
 TYPED_TEST(StorageNoInstance, SortOrdered) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> pool;
+    const std::array entity{entt::entity{16}, entt::entity{8}, entt::entity{4}, entt::entity{2}, entt::entity{1}};
 
-    entt::entity entity[5u]{entt::entity{42}, entt::entity{12}, entt::entity{9}, entt::entity{7}, entt::entity{3}}; // NOLINT
-
-    pool.insert(std::begin(entity), std::end(entity));
+    pool.insert(entity.begin(), entity.end());
     pool.sort(std::less{});
 
-    ASSERT_TRUE(std::equal(std::rbegin(entity), std::rend(entity), pool.begin(), pool.end()));
+    ASSERT_TRUE(std::equal(entity.rbegin(), entity.rend(), pool.begin(), pool.end()));
 }
 
 TYPED_TEST(StorageNoInstance, SortReverse) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> pool;
+    const std::array entity{entt::entity{1}, entt::entity{2}, entt::entity{4}, entt::entity{8}, entt::entity{16}};
 
-    entt::entity entity[5u]{entt::entity{3}, entt::entity{7}, entt::entity{9}, entt::entity{12}, entt::entity{42}}; // NOLINT
-
-    pool.insert(std::begin(entity), std::end(entity));
+    pool.insert(entity.begin(), entity.end());
     pool.sort(std::less{});
 
-    ASSERT_TRUE(std::equal(std::begin(entity), std::end(entity), pool.begin(), pool.end()));
+    ASSERT_TRUE(std::equal(entity.begin(), entity.end(), pool.begin(), pool.end()));
 }
 
 TYPED_TEST(StorageNoInstance, SortUnordered) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> pool;
+    const std::array entity{entt::entity{4}, entt::entity{2}, entt::entity{1}, entt::entity{8}, entt::entity{16}};
 
-    entt::entity entity[5u]{entt::entity{9}, entt::entity{7}, entt::entity{3}, entt::entity{12}, entt::entity{42}}; // NOLINT
-
-    pool.insert(std::begin(entity), std::end(entity));
+    pool.insert(entity.begin(), entity.end());
     pool.sort(std::less{});
 
     ASSERT_EQ(pool.data()[0u], entity[4u]);
@@ -585,14 +600,14 @@ TYPED_TEST(StorageNoInstance, SortUnordered) {
 
 TYPED_TEST(StorageNoInstance, SortN) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> pool;
+    const std::array entity{entt::entity{2}, entt::entity{4}, entt::entity{1}, entt::entity{8}, entt::entity{16}};
 
-    entt::entity entity[5u]{entt::entity{7}, entt::entity{9}, entt::entity{3}, entt::entity{12}, entt::entity{42}}; // NOLINT
-
-    pool.insert(std::begin(entity), std::end(entity));
+    pool.insert(entity.begin(), entity.end());
     pool.sort_n(0u, std::less{});
 
-    ASSERT_TRUE(std::equal(std::rbegin(entity), std::rend(entity), pool.begin(), pool.end()));
+    ASSERT_TRUE(std::equal(entity.rbegin(), entity.rend(), pool.begin(), pool.end()));
 
     pool.sort_n(2u, std::less{});
 
@@ -600,7 +615,8 @@ TYPED_TEST(StorageNoInstance, SortN) {
     ASSERT_EQ(pool.data()[1u], entity[0u]);
     ASSERT_EQ(pool.data()[2u], entity[2u]);
 
-    pool.sort_n(5u, std::less{}); // NOLINT
+    const auto length = 5u;
+    pool.sort_n(length, std::less{});
 
     ASSERT_EQ(pool.data()[0u], entity[4u]);
     ASSERT_EQ(pool.data()[1u], entity[3u]);
@@ -611,33 +627,33 @@ TYPED_TEST(StorageNoInstance, SortN) {
 
 TYPED_TEST(StorageNoInstance, SortAsDisjoint) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> lhs;
     const entt::storage<value_type> rhs;
+    const std::array entity{entt::entity{1}, entt::entity{2}, entt::entity{4}};
 
-    entt::entity lhs_entity[3u]{entt::entity{3}, entt::entity{12}, entt::entity{42}}; // NOLINT
+    lhs.insert(entity.begin(), entity.end());
 
-    lhs.insert(std::begin(lhs_entity), std::end(lhs_entity));
-
-    ASSERT_TRUE(std::equal(std::rbegin(lhs_entity), std::rend(lhs_entity), lhs.begin(), lhs.end()));
+    ASSERT_TRUE(std::equal(entity.rbegin(), entity.rend(), lhs.begin(), lhs.end()));
 
     lhs.sort_as(rhs.begin(), rhs.end());
 
-    ASSERT_TRUE(std::equal(std::rbegin(lhs_entity), std::rend(lhs_entity), lhs.begin(), lhs.end()));
+    ASSERT_TRUE(std::equal(entity.rbegin(), entity.rend(), lhs.begin(), lhs.end()));
 }
 
 TYPED_TEST(StorageNoInstance, SortAsOverlap) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> lhs;
     entt::storage<value_type> rhs;
+    const std::array lhs_entity{entt::entity{1}, entt::entity{2}, entt::entity{4}};
+    const std::array rhs_entity{entt::entity{2}};
 
-    entt::entity lhs_entity[3u]{entt::entity{3}, entt::entity{12}, entt::entity{42}}; // NOLINT
-    entt::entity rhs_entity[1u]{entt::entity{12}};                                    // NOLINT
+    lhs.insert(lhs_entity.begin(), lhs_entity.end());
+    rhs.insert(rhs_entity.begin(), rhs_entity.end());
 
-    lhs.insert(std::begin(lhs_entity), std::end(lhs_entity));
-    rhs.insert(std::begin(rhs_entity), std::end(rhs_entity));
-
-    ASSERT_TRUE(std::equal(std::rbegin(lhs_entity), std::rend(lhs_entity), lhs.begin(), lhs.end()));
-    ASSERT_TRUE(std::equal(std::rbegin(rhs_entity), std::rend(rhs_entity), rhs.begin(), rhs.end()));
+    ASSERT_TRUE(std::equal(lhs_entity.rbegin(), lhs_entity.rend(), lhs.begin(), lhs.end()));
+    ASSERT_TRUE(std::equal(rhs_entity.rbegin(), rhs_entity.rend(), rhs.begin(), rhs.end()));
 
     lhs.sort_as(rhs.begin(), rhs.end());
 
@@ -648,36 +664,36 @@ TYPED_TEST(StorageNoInstance, SortAsOverlap) {
 
 TYPED_TEST(StorageNoInstance, SortAsOrdered) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> lhs;
     entt::storage<value_type> rhs;
+    const std::array lhs_entity{entt::entity{1}, entt::entity{2}, entt::entity{4}, entt::entity{8}, entt::entity{16}};
+    const std::array rhs_entity{entt::entity{32}, entt::entity{1}, entt::entity{2}, entt::entity{4}, entt::entity{8}, entt::entity{16}};
 
-    entt::entity lhs_entity[5u]{entt::entity{1}, entt::entity{2}, entt::entity{3}, entt::entity{4}, entt::entity{5}};                  // NOLINT
-    entt::entity rhs_entity[6u]{entt::entity{6}, entt::entity{1}, entt::entity{2}, entt::entity{3}, entt::entity{4}, entt::entity{5}}; // NOLINT
+    lhs.insert(lhs_entity.begin(), lhs_entity.end());
+    rhs.insert(rhs_entity.begin(), rhs_entity.end());
 
-    lhs.insert(std::begin(lhs_entity), std::end(lhs_entity));
-    rhs.insert(std::begin(rhs_entity), std::end(rhs_entity));
-
-    ASSERT_TRUE(std::equal(std::rbegin(lhs_entity), std::rend(lhs_entity), lhs.begin(), lhs.end()));
-    ASSERT_TRUE(std::equal(std::rbegin(rhs_entity), std::rend(rhs_entity), rhs.begin(), rhs.end()));
+    ASSERT_TRUE(std::equal(lhs_entity.rbegin(), lhs_entity.rend(), lhs.begin(), lhs.end()));
+    ASSERT_TRUE(std::equal(rhs_entity.rbegin(), rhs_entity.rend(), rhs.begin(), rhs.end()));
 
     rhs.sort_as(lhs.begin(), lhs.end());
 
-    ASSERT_TRUE(std::equal(std::rbegin(rhs_entity), std::rend(rhs_entity), rhs.begin(), rhs.end()));
+    ASSERT_TRUE(std::equal(rhs_entity.rbegin(), rhs_entity.rend(), rhs.begin(), rhs.end()));
 }
 
 TYPED_TEST(StorageNoInstance, SortAsReverse) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> lhs;
     entt::storage<value_type> rhs;
+    const std::array lhs_entity{entt::entity{1}, entt::entity{2}, entt::entity{4}, entt::entity{8}, entt::entity{16}};
+    const std::array rhs_entity{entt::entity{16}, entt::entity{8}, entt::entity{4}, entt::entity{2}, entt::entity{1}, entt::entity{32}};
 
-    entt::entity lhs_entity[5u]{entt::entity{1}, entt::entity{2}, entt::entity{3}, entt::entity{4}, entt::entity{5}};                  // NOLINT
-    entt::entity rhs_entity[6u]{entt::entity{5}, entt::entity{4}, entt::entity{3}, entt::entity{2}, entt::entity{1}, entt::entity{6}}; // NOLINT
+    lhs.insert(lhs_entity.begin(), lhs_entity.end());
+    rhs.insert(rhs_entity.begin(), rhs_entity.end());
 
-    lhs.insert(std::begin(lhs_entity), std::end(lhs_entity));
-    rhs.insert(std::begin(rhs_entity), std::end(rhs_entity));
-
-    ASSERT_TRUE(std::equal(std::rbegin(lhs_entity), std::rend(lhs_entity), lhs.begin(), lhs.end()));
-    ASSERT_TRUE(std::equal(std::rbegin(rhs_entity), std::rend(rhs_entity), rhs.begin(), rhs.end()));
+    ASSERT_TRUE(std::equal(lhs_entity.rbegin(), lhs_entity.rend(), lhs.begin(), lhs.end()));
+    ASSERT_TRUE(std::equal(rhs_entity.rbegin(), rhs_entity.rend(), rhs.begin(), rhs.end()));
 
     rhs.sort_as(lhs.begin(), lhs.end());
 
@@ -691,17 +707,17 @@ TYPED_TEST(StorageNoInstance, SortAsReverse) {
 
 TYPED_TEST(StorageNoInstance, SortAsUnordered) {
     using value_type = typename TestFixture::type;
+
     entt::storage<value_type> lhs;
     entt::storage<value_type> rhs;
+    const std::array lhs_entity{entt::entity{1}, entt::entity{2}, entt::entity{4}, entt::entity{8}, entt::entity{16}};
+    const std::array rhs_entity{entt::entity{4}, entt::entity{2}, entt::entity{32}, entt::entity{1}, entt::entity{8}, entt::entity{16}};
 
-    entt::entity lhs_entity[5u]{entt::entity{1}, entt::entity{2}, entt::entity{3}, entt::entity{4}, entt::entity{5}};                  // NOLINT
-    entt::entity rhs_entity[6u]{entt::entity{3}, entt::entity{2}, entt::entity{6}, entt::entity{1}, entt::entity{4}, entt::entity{5}}; // NOLINT
+    lhs.insert(lhs_entity.begin(), lhs_entity.end());
+    rhs.insert(rhs_entity.begin(), rhs_entity.end());
 
-    lhs.insert(std::begin(lhs_entity), std::end(lhs_entity));
-    rhs.insert(std::begin(rhs_entity), std::end(rhs_entity));
-
-    ASSERT_TRUE(std::equal(std::rbegin(lhs_entity), std::rend(lhs_entity), lhs.begin(), lhs.end()));
-    ASSERT_TRUE(std::equal(std::rbegin(rhs_entity), std::rend(rhs_entity), rhs.begin(), rhs.end()));
+    ASSERT_TRUE(std::equal(lhs_entity.rbegin(), lhs_entity.rend(), lhs.begin(), lhs.end()));
+    ASSERT_TRUE(std::equal(rhs_entity.rbegin(), rhs_entity.rend(), rhs.begin(), rhs.end()));
 
     rhs.sort_as(lhs.begin(), lhs.end());
 
