@@ -1,4 +1,5 @@
 #include <cstddef>
+#include <type_traits>
 #include <utility>
 #include <gtest/gtest.h>
 #include <entt/core/hashed_string.hpp>
@@ -9,76 +10,71 @@
 #include <entt/meta/policy.hpp>
 #include <entt/meta/range.hpp>
 #include <entt/meta/resolve.hpp>
-#include "../common/config.h"
+#include "../../common/config.h"
+#include "../../common/meta_traits.h"
 
-struct base_t {
-    base_t() = default;
-    virtual ~base_t() = default;
+struct base {
+    base() = default;
+    virtual ~base() = default;
 
-    static void destroy(base_t &) {
-        ++counter;
-    }
-
-    void setter(int v) {
-        value = v;
+    void setter(int iv) {
+        value = iv;
     }
 
     [[nodiscard]] int getter() const {
         return value;
     }
 
-    static void static_setter(base_t &ref, int v) {
-        ref.value = v;
+    static void static_setter(base &ref, int iv) {
+        ref.value = iv;
     }
 
-    inline static int counter = 0; // NOLINT
     int value{3};
 };
 
-void fake_member(base_t &instance, int value) {
+void fake_member(base &instance, int value) {
     instance.value = value;
 }
 
-[[nodiscard]] int fake_const_member(const base_t &instance) {
+[[nodiscard]] int fake_const_member(const base &instance) {
     return instance.value;
 }
 
-struct derived_t: base_t {
-    derived_t()
-        : base_t{} {}
+struct derived: base {
+    derived() = default;
 };
 
-struct func_t {
-    [[nodiscard]] int f(const base_t &, int a, int b) {
-        return f(a, b);
+struct function {
+    [[nodiscard]] int f(const base &, int val, int other) {
+        return f(val, other);
     }
 
-    [[nodiscard]] int f(int a, int b) { // NOLINT
-        value = a;
-        return b * b;
+    [[nodiscard]] int f(int val, const int other) {
+        value = val;
+        return other * other;
     }
 
-    [[nodiscard]] int f(int v) const {
-        return v * v;
+    [[nodiscard]] int f(int iv) const {
+        return value * iv;
     }
 
-    void g(int v) {
-        value = v * v;
+    void g(int iv) {
+        value = iv * iv;
     }
 
-    [[nodiscard]] static int h(int &v) {
-        return (v *= value);
+    [[nodiscard]] static int h(int &iv, const function &instance) {
+        return (iv *= instance.value);
     }
 
-    static void k(int v) {
-        value = v;
+    static void k(int iv, function &instance) {
+        instance.value = iv;
     }
 
-    [[nodiscard]] int v(int v) const {
-        return (value = v);
+    [[nodiscard]] int v(int &iv) const {
+        return (iv = value);
     }
 
-    [[nodiscard]] int &a() const {
+    [[nodiscard]] int &a() {
         return value;
     }
 
@@ -86,7 +82,7 @@ struct func_t {
         return value;
     }
 
-    inline static int value = 0; // NOLINT
+    int value{};
 };
 
 double double_member(const double &value) {
@@ -101,41 +97,43 @@ struct MetaFunc: ::testing::Test {
             .type("double"_hs)
             .func<&double_member>("member"_hs);
 
-        entt::meta<base_t>()
+        entt::meta<base>()
             .type("base"_hs)
-            .dtor<base_t::destroy>()
-            .func<&base_t::setter>("setter"_hs)
+            .func<&base::setter>("setter"_hs)
             .func<fake_member>("fake_member"_hs)
             .func<fake_const_member>("fake_const_member"_hs);
 
-        entt::meta<derived_t>()
+        entt::meta<derived>()
             .type("derived"_hs)
-            .base<base_t>()
-            .func<&base_t::setter>("setter_from_base"_hs)
-            .func<&base_t::getter>("getter_from_base"_hs)
-            .func<&base_t::static_setter>("static_setter_from_base"_hs)
-            .dtor<derived_t::destroy>();
+            .base<base>()
+            .func<&base::setter>("setter_from_base"_hs)
+            .func<&base::getter>("getter_from_base"_hs)
+            .func<&base::static_setter>("static_setter_from_base"_hs);
 
-        entt::meta<func_t>()
+        entt::meta<function>()
             .type("func"_hs)
-            .func<&entt::registry::emplace_or_replace<func_t>>("emplace"_hs)
-            .func<entt::overload<int(const base_t &, int, int)>(&func_t::f)>("f3"_hs)
-            .func<entt::overload<int(int, int)>(&func_t::f)>("f2"_hs)
+            .func<&entt::registry::emplace_or_replace<function>, entt::as_ref_t>("emplace"_hs)
+            .traits(test::meta_traits::one | test::meta_traits::two | test::meta_traits::three)
+            .func<entt::overload<int(const base &, int, int)>(&function::f)>("f3"_hs)
+            .traits(test::meta_traits::three)
+            .func<entt::overload<int(int, int)>(&function::f)>("f2"_hs)
+            .traits(test::meta_traits::two)
+            .custom<int>(2)
             .prop("true"_hs, false)
-            .func<entt::overload<int(int) const>(&func_t::f)>("f1"_hs)
+            .func<entt::overload<int(int) const>(&function::f)>("f1"_hs)
+            .traits(test::meta_traits::one)
             .prop("true"_hs, false)
-            .func<&func_t::g>("g"_hs)
+            .func<&function::g>("g"_hs)
+            .custom<char>('c')
             .prop("true"_hs, false)
-            .func<func_t::h>("h"_hs)
+            .func<function::h>("h"_hs)
             .prop("true"_hs, false)
-            .func<func_t::k>("k"_hs)
+            .func<function::k>("k"_hs)
             .prop("true"_hs, false)
-            .func<&func_t::v, entt::as_void_t>("v"_hs)
-            .func<&func_t::a, entt::as_ref_t>("a"_hs)
-            .func<&func_t::a, entt::as_cref_t>("ca"_hs)
+            .func<&function::v, entt::as_void_t>("v"_hs)
+            .func<&function::a, entt::as_ref_t>("a"_hs)
+            .func<&function::a, entt::as_cref_t>("ca"_hs)
             .conv<int>();
-
-        base_t::counter = 0;
     }
 
     void TearDown() override {
@@ -145,7 +143,7 @@ struct MetaFunc: ::testing::Test {
     std::size_t reset_and_check() {
         std::size_t count = 0;
 
-        for(auto func: entt::resolve<func_t>().func()) {
+        for(auto func: entt::resolve<function>().func()) {
             for(auto curr = func.second; curr; curr = curr.next()) {
                 ++count;
             }
@@ -153,7 +151,7 @@ struct MetaFunc: ::testing::Test {
 
         SetUp();
 
-        for(auto func: entt::resolve<func_t>().func()) {
+        for(auto func: entt::resolve<function>().func()) {
             for(auto curr = func.second; curr; curr = curr.next()) {
                 --count;
             }
@@ -168,8 +166,8 @@ using MetaFuncDeathTest = MetaFunc;
 TEST_F(MetaFunc, Functionalities) {
     using namespace entt::literals;
 
-    auto func = entt::resolve<func_t>().func("f2"_hs);
-    func_t instance{};
+    auto func = entt::resolve<function>().func("f2"_hs);
+    function instance{};
 
     ASSERT_TRUE(func);
 
@@ -193,7 +191,7 @@ TEST_F(MetaFunc, Functionalities) {
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<int>());
     ASSERT_EQ(any.cast<int>(), 4);
-    ASSERT_EQ(func_t::value, 3);
+    ASSERT_EQ(instance.value, 3);
 
     for(auto curr: func.prop()) {
         ASSERT_EQ(curr.first, "true"_hs);
@@ -209,11 +207,48 @@ TEST_F(MetaFunc, Functionalities) {
     ASSERT_FALSE(prop.value().cast<bool>());
 }
 
+TEST_F(MetaFunc, UserTraits) {
+    using namespace entt::literals;
+
+    ASSERT_EQ(entt::resolve<function>().func("h"_hs).traits<test::meta_traits>(), test::meta_traits::none);
+    ASSERT_EQ(entt::resolve<function>().func("k"_hs).traits<test::meta_traits>(), test::meta_traits::none);
+
+    ASSERT_EQ(entt::resolve<function>().func("emplace"_hs).traits<test::meta_traits>(), test::meta_traits::one | test::meta_traits::two | test::meta_traits::three);
+    ASSERT_EQ(entt::resolve<function>().func("f1"_hs).traits<test::meta_traits>(), test::meta_traits::one);
+    ASSERT_EQ(entt::resolve<function>().func("f2"_hs).traits<test::meta_traits>(), test::meta_traits::two);
+    ASSERT_EQ(entt::resolve<function>().func("f3"_hs).traits<test::meta_traits>(), test::meta_traits::three);
+}
+
+ENTT_DEBUG_TEST_F(MetaFuncDeathTest, UserTraits) {
+    using namespace entt::literals;
+
+    using traits_type = entt::internal::meta_traits;
+    constexpr auto value = traits_type{static_cast<std::underlying_type_t<traits_type>>(traits_type::_user_defined_traits) + 1u};
+    ASSERT_DEATH(entt::meta<function>().func<&function::g>("g"_hs).traits(value), "");
+}
+
+TEST_F(MetaFunc, Custom) {
+    using namespace entt::literals;
+
+    ASSERT_EQ(*static_cast<const char *>(entt::resolve<function>().func("g"_hs).custom()), 'c');
+    ASSERT_EQ(static_cast<const char &>(entt::resolve<function>().func("g"_hs).custom()), 'c');
+
+    ASSERT_EQ(static_cast<const int *>(entt::resolve<function>().func("g"_hs).custom()), nullptr);
+    ASSERT_EQ(static_cast<const int *>(entt::resolve<function>().func("h"_hs).custom()), nullptr);
+}
+
+ENTT_DEBUG_TEST_F(MetaFuncDeathTest, Custom) {
+    using namespace entt::literals;
+
+    ASSERT_DEATH([[maybe_unused]] int value = entt::resolve<function>().func("g"_hs).custom(), "");
+    ASSERT_DEATH([[maybe_unused]] char value = entt::resolve<function>().func("h"_hs).custom(), "");
+}
+
 TEST_F(MetaFunc, Const) {
     using namespace entt::literals;
 
-    auto func = entt::resolve<func_t>().func("f1"_hs);
-    func_t instance{};
+    auto func = entt::resolve<function>().func("f1"_hs);
+    function instance{2};
 
     ASSERT_TRUE(func);
     ASSERT_EQ(func.arity(), 1u);
@@ -224,12 +259,12 @@ TEST_F(MetaFunc, Const) {
     ASSERT_FALSE(func.arg(1u));
 
     auto any = func.invoke(instance, 4);
-    auto empty = func.invoke(instance, derived_t{});
+    auto empty = func.invoke(instance, derived{});
 
     ASSERT_FALSE(empty);
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<int>());
-    ASSERT_EQ(any.cast<int>(), 16);
+    ASSERT_EQ(any.cast<int>(), 8);
 
     for(auto curr: func.prop()) {
         ASSERT_EQ(curr.first, "true"_hs);
@@ -248,8 +283,8 @@ TEST_F(MetaFunc, Const) {
 TEST_F(MetaFunc, RetVoid) {
     using namespace entt::literals;
 
-    auto func = entt::resolve<func_t>().func("g"_hs);
-    func_t instance{};
+    auto func = entt::resolve<function>().func("g"_hs);
+    function instance{};
 
     ASSERT_TRUE(func);
     ASSERT_EQ(func.arity(), 1u);
@@ -259,11 +294,11 @@ TEST_F(MetaFunc, RetVoid) {
     ASSERT_EQ(func.arg(0u), entt::resolve<int>());
     ASSERT_FALSE(func.arg(1u));
 
-    auto any = func.invoke(instance, 5); // NOLINT
+    auto any = func.invoke(instance, 4);
 
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<void>());
-    ASSERT_EQ(func_t::value, 25);
+    ASSERT_EQ(instance.value, 16);
 
     for(auto curr: func.prop()) {
         ASSERT_EQ(curr.first, "true"_hs);
@@ -282,19 +317,20 @@ TEST_F(MetaFunc, RetVoid) {
 TEST_F(MetaFunc, Static) {
     using namespace entt::literals;
 
-    auto func = entt::resolve<func_t>().func("h"_hs);
-    func_t::value = 2;
+    auto func = entt::resolve<function>().func("h"_hs);
+    function instance{2};
 
     ASSERT_TRUE(func);
-    ASSERT_EQ(func.arity(), 1u);
+    ASSERT_EQ(func.arity(), 2u);
     ASSERT_FALSE(func.is_const());
     ASSERT_TRUE(func.is_static());
     ASSERT_EQ(func.ret(), entt::resolve<int>());
     ASSERT_EQ(func.arg(0u), entt::resolve<int>());
-    ASSERT_FALSE(func.arg(1u));
+    ASSERT_EQ(func.arg(1u), entt::resolve<function>());
+    ASSERT_FALSE(func.arg(2u));
 
-    auto any = func.invoke({}, 3);
-    auto empty = func.invoke({}, derived_t{});
+    auto any = func.invoke({}, 3, entt::forward_as_meta(instance));
+    auto empty = func.invoke({}, derived{}, entt::forward_as_meta(instance));
 
     ASSERT_FALSE(empty);
     ASSERT_TRUE(any);
@@ -318,21 +354,23 @@ TEST_F(MetaFunc, Static) {
 TEST_F(MetaFunc, StaticRetVoid) {
     using namespace entt::literals;
 
-    auto func = entt::resolve<func_t>().func("k"_hs);
+    auto func = entt::resolve<function>().func("k"_hs);
+    function instance{};
 
     ASSERT_TRUE(func);
-    ASSERT_EQ(func.arity(), 1u);
+    ASSERT_EQ(func.arity(), 2u);
     ASSERT_FALSE(func.is_const());
     ASSERT_TRUE(func.is_static());
     ASSERT_EQ(func.ret(), entt::resolve<void>());
     ASSERT_EQ(func.arg(0u), entt::resolve<int>());
-    ASSERT_FALSE(func.arg(1u));
+    ASSERT_EQ(func.arg(1u), entt::resolve<function>());
+    ASSERT_FALSE(func.arg(2u));
 
-    auto any = func.invoke({}, 42); // NOLINT
+    auto any = func.invoke({}, 3, entt::forward_as_meta(instance));
 
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<void>());
-    ASSERT_EQ(func_t::value, 42);
+    ASSERT_EQ(instance.value, 3);
 
     for(auto curr: func.prop()) {
         ASSERT_EQ(curr.first, "true"_hs);
@@ -351,9 +389,9 @@ TEST_F(MetaFunc, StaticRetVoid) {
 TEST_F(MetaFunc, StaticAsMember) {
     using namespace entt::literals;
 
-    base_t instance{};
-    auto func = entt::resolve<base_t>().func("fake_member"_hs);
-    auto any = func.invoke(instance, 42); // NOLINT
+    base instance{};
+    auto func = entt::resolve<base>().func("fake_member"_hs);
+    auto any = func.invoke(instance, 3);
 
     ASSERT_TRUE(func);
     ASSERT_EQ(func.arity(), 1u);
@@ -365,19 +403,19 @@ TEST_F(MetaFunc, StaticAsMember) {
 
     ASSERT_EQ(func.prop().cbegin(), func.prop().cend());
 
-    ASSERT_FALSE(func.invoke({}, 42));
-    ASSERT_FALSE(func.invoke(std::as_const(instance), 42));
+    ASSERT_FALSE(func.invoke({}, 3));
+    ASSERT_FALSE(func.invoke(std::as_const(instance), 3));
 
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<void>());
-    ASSERT_EQ(instance.value, 42);
+    ASSERT_EQ(instance.value, 3);
 }
 
 TEST_F(MetaFunc, StaticAsConstMember) {
     using namespace entt::literals;
 
-    base_t instance{};
-    auto func = entt::resolve<base_t>().func("fake_const_member"_hs);
+    base instance{};
+    auto func = entt::resolve<base>().func("fake_const_member"_hs);
     auto any = func.invoke(std::as_const(instance));
 
     ASSERT_TRUE(func);
@@ -424,8 +462,8 @@ TEST_F(MetaFunc, NonClassTypeMember) {
 TEST_F(MetaFunc, MetaAnyArgs) {
     using namespace entt::literals;
 
-    func_t instance;
-    auto any = entt::resolve<func_t>().func("f1"_hs).invoke(instance, 3);
+    function instance{3};
+    auto any = entt::resolve<function>().func("f1"_hs).invoke(instance, 3);
 
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<int>());
@@ -436,15 +474,15 @@ TEST_F(MetaFunc, InvalidArgs) {
     using namespace entt::literals;
 
     int value = 3;
-    ASSERT_FALSE(entt::resolve<func_t>().func("f1"_hs).invoke(value, 'c'));
+    ASSERT_FALSE(entt::resolve<function>().func("f1"_hs).invoke(value, 'c'));
 }
 
 TEST_F(MetaFunc, CastAndConvert) {
     using namespace entt::literals;
 
-    func_t instance;
+    function instance;
     instance.value = 3;
-    auto any = entt::resolve<func_t>().func("f3"_hs).invoke(instance, derived_t{}, 0, instance);
+    auto any = entt::resolve<function>().func("f3"_hs).invoke(instance, derived{}, 0, instance);
 
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<int>());
@@ -455,8 +493,8 @@ TEST_F(MetaFunc, CastAndConvert) {
 TEST_F(MetaFunc, ArithmeticConversion) {
     using namespace entt::literals;
 
-    func_t instance;
-    auto any = entt::resolve<func_t>().func("f2"_hs).invoke(instance, true, 4.2);
+    function instance;
+    auto any = entt::resolve<function>().func("f2"_hs).invoke(instance, true, 4.2);
 
     ASSERT_TRUE(any);
     ASSERT_EQ(any.type(), entt::resolve<int>());
@@ -467,13 +505,14 @@ TEST_F(MetaFunc, ArithmeticConversion) {
 TEST_F(MetaFunc, ArgsByRef) {
     using namespace entt::literals;
 
-    auto func = entt::resolve<func_t>().func("h"_hs);
-    func_t::value = 2;
+    auto func = entt::resolve<function>().func("h"_hs);
+
+    function instance{2};
     entt::meta_any any{3};
     int value = 4;
 
-    ASSERT_EQ(func.invoke({}, entt::forward_as_meta(value)).cast<int>(), 8);
-    ASSERT_EQ(func.invoke({}, any.as_ref()).cast<int>(), 6);
+    ASSERT_EQ(func.invoke({}, entt::forward_as_meta(value), entt::forward_as_meta(instance)).cast<int>(), 8);
+    ASSERT_EQ(func.invoke({}, any.as_ref(), entt::forward_as_meta(instance)).cast<int>(), 6);
     ASSERT_EQ(any.cast<int>(), 6);
     ASSERT_EQ(value, 8);
 }
@@ -481,25 +520,25 @@ TEST_F(MetaFunc, ArgsByRef) {
 TEST_F(MetaFunc, ArgsByConstRef) {
     using namespace entt::literals;
 
-    func_t instance{};
-    auto func = entt::resolve<func_t>().func("g"_hs);
+    function instance{};
+    auto func = entt::resolve<function>().func("g"_hs);
     entt::meta_any any{2};
     int value = 3;
 
     ASSERT_TRUE(func.invoke(instance, entt::forward_as_meta(std::as_const(value))));
-    ASSERT_EQ(func_t::value, 9);
+    ASSERT_EQ(instance.value, 9);
 
     ASSERT_TRUE(func.invoke(instance, std::as_const(any).as_ref()));
-    ASSERT_EQ(func_t::value, 4);
+    ASSERT_EQ(instance.value, 4);
 }
 
 TEST_F(MetaFunc, ConstInstance) {
     using namespace entt::literals;
 
-    func_t instance{};
-    auto any = entt::resolve<func_t>().func("f1"_hs).invoke(std::as_const(instance), 2);
+    function instance{2};
+    auto any = entt::resolve<function>().func("f1"_hs).invoke(std::as_const(instance), 2);
 
-    ASSERT_FALSE(entt::resolve<func_t>().func("g"_hs).invoke(std::as_const(instance), 42));
+    ASSERT_FALSE(entt::resolve<function>().func("g"_hs).invoke(std::as_const(instance), 1));
     ASSERT_TRUE(any);
     ASSERT_EQ(any.cast<int>(), 4);
 }
@@ -507,19 +546,20 @@ TEST_F(MetaFunc, ConstInstance) {
 TEST_F(MetaFunc, AsVoid) {
     using namespace entt::literals;
 
-    auto func = entt::resolve<func_t>().func("v"_hs);
-    func_t instance{};
+    auto func = entt::resolve<function>().func("v"_hs);
+    function instance{3};
+    int value{2};
 
-    ASSERT_EQ(func.invoke(instance, 42), entt::meta_any{std::in_place_type<void>});
+    ASSERT_EQ(func.invoke(instance, entt::forward_as_meta(value)), entt::meta_any{std::in_place_type<void>});
     ASSERT_EQ(func.ret(), entt::resolve<void>());
-    ASSERT_EQ(instance.value, 42);
+    ASSERT_EQ(value, instance.value);
 }
 
 TEST_F(MetaFunc, AsRef) {
     using namespace entt::literals;
 
-    func_t instance{};
-    auto func = entt::resolve<func_t>().func("a"_hs);
+    function instance{};
+    auto func = entt::resolve<function>().func("a"_hs);
     func.invoke(instance).cast<int &>() = 3;
 
     ASSERT_EQ(func.ret(), entt::resolve<int>());
@@ -529,8 +569,8 @@ TEST_F(MetaFunc, AsRef) {
 TEST_F(MetaFunc, AsConstRef) {
     using namespace entt::literals;
 
-    func_t instance{};
-    auto func = entt::resolve<func_t>().func("ca"_hs);
+    function instance{3};
+    auto func = entt::resolve<function>().func("ca"_hs);
 
     ASSERT_EQ(func.ret(), entt::resolve<int>());
     ASSERT_EQ(func.invoke(instance).cast<const int &>(), 3);
@@ -540,8 +580,8 @@ TEST_F(MetaFunc, AsConstRef) {
 ENTT_DEBUG_TEST_F(MetaFuncDeathTest, AsConstRef) {
     using namespace entt::literals;
 
-    func_t instance{};
-    auto func = entt::resolve<func_t>().func("ca"_hs);
+    function instance{};
+    auto func = entt::resolve<function>().func("ca"_hs);
 
     ASSERT_DEATH((func.invoke(instance).cast<int &>() = 3), "");
 }
@@ -549,41 +589,41 @@ ENTT_DEBUG_TEST_F(MetaFuncDeathTest, AsConstRef) {
 TEST_F(MetaFunc, InvokeBaseFunction) {
     using namespace entt::literals;
 
-    auto type = entt::resolve<derived_t>();
-    derived_t instance{};
+    auto type = entt::resolve<derived>();
+    derived instance{};
 
     ASSERT_TRUE(type.func("setter"_hs));
     ASSERT_EQ(instance.value, 3);
 
-    type.func("setter"_hs).invoke(instance, 42); // NOLINT
+    type.func("setter"_hs).invoke(instance, 1);
 
-    ASSERT_EQ(instance.value, 42);
+    ASSERT_EQ(instance.value, 1);
 }
 
 TEST_F(MetaFunc, InvokeFromBase) {
     using namespace entt::literals;
 
-    auto type = entt::resolve<derived_t>();
-    derived_t instance{};
+    auto type = entt::resolve<derived>();
+    derived instance{};
 
     auto setter_from_base = type.func("setter_from_base"_hs);
 
     ASSERT_TRUE(setter_from_base);
     ASSERT_EQ(instance.value, 3);
 
-    setter_from_base.invoke(instance, 42); // NOLINT
+    setter_from_base.invoke(instance, 1);
 
-    ASSERT_EQ(instance.value, 42);
+    ASSERT_EQ(instance.value, 1);
 
     auto getter_from_base = type.func("getter_from_base"_hs);
 
     ASSERT_TRUE(getter_from_base);
-    ASSERT_EQ(getter_from_base.invoke(instance).cast<int>(), 42);
+    ASSERT_EQ(getter_from_base.invoke(instance).cast<int>(), 1);
 
     auto static_setter_from_base = type.func("static_setter_from_base"_hs);
 
     ASSERT_TRUE(static_setter_from_base);
-    ASSERT_EQ(instance.value, 42);
+    ASSERT_EQ(instance.value, 1);
 
     static_setter_from_base.invoke(instance, 3);
 
@@ -593,13 +633,13 @@ TEST_F(MetaFunc, InvokeFromBase) {
 TEST_F(MetaFunc, ExternalMemberFunction) {
     using namespace entt::literals;
 
-    auto func = entt::resolve<func_t>().func("emplace"_hs);
+    auto func = entt::resolve<function>().func("emplace"_hs);
 
     ASSERT_TRUE(func);
     ASSERT_EQ(func.arity(), 2u);
     ASSERT_FALSE(func.is_const());
     ASSERT_TRUE(func.is_static());
-    ASSERT_EQ(func.ret(), entt::resolve<void>());
+    ASSERT_EQ(func.ret(), entt::resolve<function>());
     ASSERT_EQ(func.arg(0u), entt::resolve<entt::registry>());
     ASSERT_EQ(func.arg(1u), entt::resolve<entt::entity>());
     ASSERT_FALSE(func.arg(2u));
@@ -607,11 +647,66 @@ TEST_F(MetaFunc, ExternalMemberFunction) {
     entt::registry registry;
     const auto entity = registry.create();
 
-    ASSERT_FALSE(registry.all_of<func_t>(entity));
+    ASSERT_FALSE(registry.all_of<function>(entity));
 
     func.invoke({}, entt::forward_as_meta(registry), entity);
 
-    ASSERT_TRUE(registry.all_of<func_t>(entity));
+    ASSERT_TRUE(registry.all_of<function>(entity));
+}
+
+TEST_F(MetaFunc, Overloaded) {
+    using namespace entt::literals;
+
+    auto type = entt::resolve<function>();
+
+    ASSERT_FALSE(type.func("f2"_hs).next());
+
+    entt::meta<function>()
+        // this should not overwrite traits and custom data
+        .func<entt::overload<int(int, int)>(&function::f)>("f2"_hs)
+        // this should put traits and custom data on the new overload instead
+        .func<entt::overload<int(int) const>(&function::f)>("f2"_hs)
+        .traits(test::meta_traits::three)
+        .custom<int>(3);
+
+    ASSERT_TRUE(type.func("f2"_hs).next());
+    ASSERT_FALSE(type.func("f2"_hs).next().next());
+
+    ASSERT_EQ(type.func("f2"_hs).traits<test::meta_traits>(), test::meta_traits::two);
+    ASSERT_EQ(type.func("f2"_hs).next().traits<test::meta_traits>(), test::meta_traits::three);
+
+    ASSERT_NE(static_cast<const int *>(type.func("f2"_hs).custom()), nullptr);
+    ASSERT_NE(static_cast<const int *>(type.func("f2"_hs).next().custom()), nullptr);
+
+    ASSERT_EQ(static_cast<const int &>(type.func("f2"_hs).custom()), 2);
+    ASSERT_EQ(static_cast<const int &>(type.func("f2"_hs).next().custom()), 3);
+}
+
+TEST_F(MetaFunc, OverloadedOrder) {
+    using namespace entt::literals;
+
+    entt::meta<function>()
+        .func<entt::overload<int(int, int)>(&function::f)>("f2"_hs)
+        .func<entt::overload<int(int) const>(&function::f)>("f2"_hs);
+
+    auto type = entt::resolve<function>();
+    auto func = type.func("f2"_hs);
+
+    ASSERT_TRUE(func);
+    ASSERT_EQ(func.arity(), 2u);
+    ASSERT_FALSE(func.is_const());
+    ASSERT_EQ(func.ret(), entt::resolve<int>());
+
+    func = func.next();
+
+    ASSERT_TRUE(func);
+    ASSERT_EQ(func.arity(), 1u);
+    ASSERT_TRUE(func.is_const());
+    ASSERT_EQ(func.ret(), entt::resolve<int>());
+
+    func = func.next();
+
+    ASSERT_FALSE(func);
 }
 
 TEST_F(MetaFunc, ReRegistration) {
@@ -619,8 +714,8 @@ TEST_F(MetaFunc, ReRegistration) {
 
     ASSERT_EQ(reset_and_check(), 0u);
 
-    func_t instance{};
-    auto type = entt::resolve<func_t>();
+    function instance{};
+    auto type = entt::resolve<function>();
 
     ASSERT_TRUE(type.func("f2"_hs));
     ASSERT_FALSE(type.invoke("f2"_hs, instance, 0));
@@ -630,9 +725,9 @@ TEST_F(MetaFunc, ReRegistration) {
     ASSERT_TRUE(type.invoke("f1"_hs, instance, 0));
     ASSERT_FALSE(type.invoke("f1"_hs, instance, 0, 0));
 
-    entt::meta<func_t>()
-        .func<entt::overload<int(int, int)>(&func_t::f)>("f"_hs)
-        .func<entt::overload<int(int) const>(&func_t::f)>("f"_hs);
+    entt::meta<function>()
+        .func<entt::overload<int(int, int)>(&function::f)>("f"_hs)
+        .func<entt::overload<int(int) const>(&function::f)>("f"_hs);
 
     ASSERT_TRUE(type.func("f1"_hs));
     ASSERT_TRUE(type.func("f2"_hs));
