@@ -1,12 +1,13 @@
 #include <cr.h>
 #include <entt/signal/dispatcher.hpp>
-#include "../common/types.h"
+#include "../../../common/boxed_type.h"
+#include "../../../common/empty.h"
 
 CR_EXPORT int cr_main(cr_plugin *ctx, cr_op operation) {
     switch(operation) {
     case CR_STEP:
-        static_cast<entt::dispatcher *>(ctx->userdata)->trigger<event>();
-        static_cast<entt::dispatcher *>(ctx->userdata)->trigger(message{42});
+        static_cast<entt::dispatcher *>(ctx->userdata)->trigger<test::empty>();
+        static_cast<entt::dispatcher *>(ctx->userdata)->trigger(test::boxed_int{4});
         break;
     case CR_CLOSE:
     case CR_LOAD:
